#!/usr/bin/env node

/**
 * Compile files.
 */

'use strict'

process.chdir(`${__dirname}/..`)

const apeTasking = require('ape-tasking')
const fs = require('fs')
const apeCompiling = require('ape-compiling')
const filecopy = require('filecopy')
const co = require('co')
const coz = require('coz')

apeTasking.runTasks('compile', [
  () => co(function * () {
    let src = 'node_modules/font-awesome/fonts/*.*'
    let dest = 'asset/fonts'
    let results = yield filecopy(src, dest)
    Object.keys(results).forEach((filename) => {
      console.log(`File generated: ${filename}`)
    })
  }),
  () => co(function * () {
    let src = 'node_modules/font-awesome/css/font-awesome.css'
    let dest = 'asset/css/font-awesome.css'
    yield filecopy(src, dest, { mkdirp: true })
  }),
  () => co(function * () {
    let src = 'asset/css/font-awesome.css'
    let dest = 'asset/css/font-awesome-inline.css'
    let result = yield apeCompiling.inlineFontCss(src, dest)
    if (!result.skipped) {
      console.log(`File generated: ${dest}`)
    }
  }),
  () => co(function * () {
    let libDir = `${__dirname}/../lib`
    return yield apeCompiling.compileReactJsx('*.jsx', {
      cwd: libDir,
      out: libDir,
      map: 'inline'
    })
  }),
  () => co(function * () {
    let demoDir = `${__dirname}/../doc/demo`
    if (!fs.existsSync(demoDir)) {
      return
    }
    yield apeCompiling.compileReactJsx('*.jsx', {
      cwd: demoDir,
      out: demoDir,
      map: 'inline',
      minified: true
    })
    yield coz.render(demoDir + '/.*.bud')
    yield apeCompiling.browserifyJs(
      `${demoDir}/demo.browser.js`,
      `${demoDir}/demo.js`,
      {
        debug: true,
        external: require('apeman-asset-javascripts/src/demo.external.json')
      })
    yield filecopy(
      require.resolve('apeman-asset-javascripts/dist/demo.external.cc.js'),
      demoDir + '/demo.external.cc.js'
    )
  })
], true)
